; Unselect word wrap

; The intent of this program is to make CrASH return a value from 0-255 instead of just 0-127.

; The following routine does NOT run as an interrupt and only generates a value when called.
; This routine is already programmed into the shell. It is possible to change the routine
; as long as it fits within 23 bytes of memory which it occupies.

; ADDR_RAND_8D8Bh:
; C3 E2 8F	JP	$8FE2		; This is the routine called by the user
;					; It jumps to the actual routine since other code
;					; exists in between.  It also allows programs to be
;					; consistantly compatible without requiring reprogramming
;
; ADDR_8FE2h:				; Routine from here is 23 bytes
; 3E 93		LD	A, $??		; Determined from code below
; E5		PUSH	HL		; Save this in case the user has something stored
; 21 LL HH	LD	HL, $HHLL	; The value here is dependent on the code below
;					; It is labeled to show how H and L are stored in memory
; 2C		INC	HL
; 25		DEC	H
; 86		ADD	A, (HL)
; 22 E6 8F	LD	($8FE6), HL
; 07		RLCA			; Does rotation affect randomness?
; 07		RLCA			; At all?
; 94		SUB	H
; 85		ADD	A, L
; E1		POP	HL		; Give back value to the user/programmer
; 32 E3 8F	LD	($8FE3), A	; This value is stored at 8FE3h
; CB 3F		SRL	A		; <-- Destroys MSb.  Why destroy MSb?
					; Now the programmer has a random value from 0-127 instead of 0-255
; C9		RET			; Return to program with PSEUDO random value

	; 8FE3h is affected by 8FE6h and 8FE7h
	; The contents of 8FE6h increments by one each key press or when called
	; The contents of 8FE7h decrements by one each key press or when called

; The following routine is what the code looks like once it is modified.  The goal is to
; Get rid of 'SRL' and replace it with a RET instruction.  This saves 10 clock cycles
; to the user/programmer and gives the programmer twice as many values to work with than
; previously.  Here it is.


; For some reason, the author of CrASH decided to have the routine return a value between
; 0-127 instead of 0-255; and I have no idea why.  The most significant bit is destroyed
; by shifting the accumulator right one bit.  To prevent this, the previous code was
; changed to the following.

; ADDR_8FE2h:				; Routine from here is 21 bytes
; 3E 93		LD	A, $??		; <-- ??   = 8FE3h
; E5		PUSH	HL		; Save value for the user
; 21 LL HH	LD	HL, HHLLh	; <-- HHLL = 8FE6h
; 2C		INC	HL
; 25		DEC	H
; 86		ADD	A, (HL)
; 22 E6 8F	LD	($8FE6), HL
; 07		RLCA
; 07		RLCA
; 94		SUB	H
; 85		ADD	A, L
; E1		POP	HL		; Return the value to the user.
; 32 E3 8F	LD	($8FE3), A
; C9		RET			; !MODIFIED BY PATCH00! Save some clocks (10 to be exact)
; 3F		CCF			; This instruction really doesn't exist when CBh exists prehand
; C9		RET

; When CB 3F is altered to C9 3F, it is no longer a single instruction.  In this case,
; C9 becomes the opcode for a return instruction (RET) and 3F becomes the opcode for
; an instruction that complements the carry flag (CCF).  It does not matter what occurs
; after the return instruction since the routine will not execute in that space.
; This value can be changed back to CBh and will again be one instruction.

; This routine could possibly be written better if it was ran as an interrupt.
; Since interrupts are called anywhere from 50 to 200 times a second (depending
; on how long it takes for an interrupt routine to execute).
; I will experiment with this using an interrupt, but first... Here is the routine
; I wrote that alters the shell code to return a random value from 0-255
; rather than 0-127.

; The comments may be taken out of this routine if chosen so.  This routine is tested
; and DOES work correctly.

; The light version is a simple program that requires no input from the user.

; #include "crash82.inc"
; .DB "Patch #00 Light",$00	; I'll be naming patches by numbers
;
;	ROM_CALL(CLEARLCD)	; Clear ONLY the screen
;	LD	HL, $8FF6	; This is the address of the instruction to change (SRL)
;	LD	A, (HL)		; Load the value for this instruction (SRL)
;	CP	$C9		; If it is not C9h (C9h = RET) then change it
;	JR	NZ, FIX		; Go fix it to C9h
;	LD	HL, ERROR	; This means it is already fixed and we need not touch it
;	ROM_CALL(D_ZT_STR)	; Display the error message (Zero Terminated)
;	JR	END		; Go end the program
; FIX:
;	LD	A, $C9		; C9h is the opcode for the RET instruction
;	LD	($8FF6), A	; 8FF6h is the location to insert the byte
;	LD	HL, ERROR-1	; 18h is the length of the message to display
;	ROM_CALL(D_LT_STR)	; Display the message (Length Terminated)
; END:
;	CALL	CR_KHAND	; Sit and wait here until the user presses a key
;	RET			; Quit the program when the user does press a key
;
;	.DB $18			; Address of ERROR+18h is where this message ends
; ERROR:
;	; ** 1234567890123456	The screen is 16 spaces wide
;	.DB "CrASH has been  "
;	.DB "modified already",$00

; The following code does the same as above except that it is more friendly and
; flexible for the user.  It allows the user to patch or unpatch the shell.
; There are most likely programs out there that already work around the current
; code the CrASH (the shell for the TI82) has.

; It is recommended that you only use this code for your application.  By allowing
; this patch to remain in memory while other programs use the RAND function may
; cause errors.

; This is a bit of long code but at least it's there.  The full patch interface.

#include "crash82.inc"
.DB "Patch #00",$00		; I'll be naming patches by numbers
				; 00 - RAND


	ROM_CALL(CLEARLCD)	; Clear ONLY the screen
	LD	HL, OPTIONS	; Load the address of the options to give the user
	ROM_CALL(D_ZT_STR)	; Display the options onto the screen
KEY_LOOP:
	CALL	GET_KEY		; Check for a key press. No press? Continue anyways
	CP	G_1		; Was it the number '1' on the keypad?
	JR	Z, OPTION_1	; Goto first option if it was
	CP	G_2		; Was it the number '2' on the keypad?
	JR	Z, OPTION_2	; Goto second option if it was
	CP	G_3		; Was it the number '3' on the keypad?
	JR	NZ, KEY_LOOP	; If not go back and check for a key press again
	RET			; Quit the program if it was 
OPTION_1:
	LD	HL, $8FF6	; This is the address of the instruction to change (SRL)
	LD	A, (HL)		; Load the value for this instruction (SRL)
	CP	$C9		; If it is not C9h (C9h = RET) then change it
	JR	Z, FIXED	; If it is fixed then get ready to quit the program
	LD	A, $C9		; C9h is the opcode for the RET instruction
	LD	($8FF6), A	; 8FF6h is the location to insert the byte
	JR	DONE		; Now I'm done, let's quit the program
OPTION_2:
	LD	HL, $8FF6	; This is the address of the instruction to change (RET)
	LD	A, (HL)		; Load the value for this instruction (RET)
	CP	$CB		; If it is not CBh (CBh = SRL) then change it
	JR	Z, FIXED	; If it is fixed then get ready to quit the program
	LD	A, $CB		; CBh is the opcode for the SRL instruction
	LD	($8FF6), A	; 8FF6h is the location of insert the byte
DONE:
	CALL	GET_HL		; Reset all coordnates to display a string properly
	ROM_CALL(D_LT_STR)	; Tell the user it has now been modified
	JR	FINE		; Get out of here already, sheesh
FIXED:
	CALL	GET_HL		; Reset all coordnates to display a string properly
	INC	HL		; Increment by one byte, we don't want to see ASCII 18h
	ROM_CALL(D_ZT_STR)	; Display the Zero Terminated string
FINE:
	CALL	CR_KHAND	; Sit and wait for the user to press a key
	RET			; Now quit the program if a key was pressed
GET_HL:
	ROM_CALL(CLEARLCD)	; Clear ONLY the screen
	LD	HL, $0000	; Start at Column 00, Row 00
	LD	(CURSOR_POS), HL; Load the coordnates into the cursor position memory
	LD	HL, ERROR-1	; Address of string minus one and load the result into HL
	RET			; Return from this back to routine

	.DB $18
ERROR:
	; ** 1234567890123456	; Screen is 16 spaces wide
	.DB "CrASH has been  "
	.DB "modified already",$00
OPTIONS:
	.DB "Patch #00       "
	.DB "1:Patch RAND    "
	.DB "2:Unpatch RAND  "
	.DB "3:Quit",$00
.end